/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JMethod.java 4078 2008-10-05 18:11:58Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.scan.api.metadata.structures;

import java.util.Arrays;

/**
 * This class defines a Method object. It is not based on reflection but allows
 * to build a JMethod based on java.lang.reflect.Method
 * @author Florent Benoit
 */
public final class JMethod {

    /**
     * Name of the method.
     */
    private String name = null;

    /**
     * Access mode (see {@link org.ow2.easybeans.asm.Opcodes}).
     */
    private int access;

    /**
     * Method's descriptor.
     */
    private String descriptor = null;

    /**
     * Method's signature.
     */
    private String signature;

    /**
     * Exceptions of the method.
     */
    private final String[] exceptions;

    /**
     * Constructor.
     * @param access the access mode (see {@link org.ow2.easybeans.asm.Opcodes})
     * @param name the method's name.
     * @param descriptor the method's descriptor (see
     *        {@link org.ow2.util.scan.api.IType Type}).
     * @param signature the method's signature. May be <tt>null</tt> if the
     *        method parameters, return type and exceptions do not use generic
     *        types.
     * @param exceptions the internal names of the method's exception classes
     *        (see
     *        {@link org.ow2.util.scan.api.IType#getInternalName() getInternalName}).
     *        May be <tt>null</tt>.
     */
    public JMethod(final int access, final String name, final String descriptor, final String signature,
            final String[] exceptions) {
        this.access = access;
        this.name = name;
        this.descriptor = descriptor;
        this.signature = signature;
        String[] tmpExceptions = exceptions;
        if (tmpExceptions != null) {
            this.exceptions = tmpExceptions.clone();
        } else {
            this.exceptions = null;
        }
    }

    /**
     * @return the access mode (see {@link org.ow2.easybeans.asm.Opcodes})
     */
    public int getAccess() {
        return access;
    }

    /**
     * Indicates whether some other object is "equal to" this one.
     * @param obj object to compare
     * @return true if given object is equals
     */
    @Override
    public boolean equals(final Object obj) {
        if (obj != null && obj instanceof JMethod) {
            JMethod other = (JMethod) obj;

            // same name
            if (!this.name.equals(other.name)) {
                return false;
            }

            // same descriptor
            if ((this.descriptor != null) && (!this.descriptor.equals(other.descriptor))) {
                return false;
            }

            // Don't check signature (which include generics information)
            // For example void method(List<Integer>) and
            //             void method(List<String>)
            // and even if signature is different, they have same erasure
            // This is not allowed, so don't need to check signature information.


            // if all tests succeed, return true
            return true;
        }
        return false;
    }

    /**
     * @return a hash code value for the object.
     */
    @Override
    public int hashCode() {
        return name.hashCode();
    }

    /**
     * @return method descriptor
     */
    public String getDescriptor() {
        return descriptor;
    }

    /**
     * @return method exceptions
     */
    public String[] getExceptions() {
        if (exceptions != null) {
            return exceptions.clone();
        }
        return null;
    }

    /**
     * @return method name
     */
    public String getName() {
        return name;
    }

    /**
     * @return method signature
     */
    public String getSignature() {
        return signature;
    }

    /**
     * @return string representation
     */
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        // classname
        sb.append(this.getClass().getName().substring(this.getClass().getPackage().getName().length() + 1));

        // name
        sb.append("[name=");
        sb.append(name);

        // access
        sb.append(", access=");
        sb.append(access);

        // descriptor
        if (descriptor != null) {
            sb.append(", descriptor=");
            sb.append(descriptor);
        }

        // signature
        if (signature != null) {
            sb.append(", signature=");
            sb.append(signature);
        }

        // exceptions
        if (exceptions != null) {
            sb.append(", exceptions=");
            sb.append(Arrays.asList(exceptions));
        }
        sb.append("]");
        return sb.toString();
    }
}
